@@ -66,10 +66,6 @@ Functions
 .. autofunction:: augment_coordinates(coordinates, box, r)
 .. autofunction:: undo_augment(results, translation, nreal)
 """
-from __future__ import division, absolute_import
-from six.moves import range
-from six import raise_from
-
 import numpy as np
 from numpy.lib.utils import deprecate
 
@@ -77,6 +73,8 @@ from .util import check_coords, check_box
 from .mdamath import triclinic_vectors
 from ._augment import augment_coordinates, undo_augment
 from .nsgrid import FastNS
+from .c_distances import _minimize_vectors_ortho, _minimize_vectors_triclinic
+
 
 # hack to select backend with backend=<backend> kwarg. Note that
 # the cython parallel code (prange) in parallel.distances is
@@ -100,16 +98,15 @@ def _run(funcname, args=None, kwargs=None, backend="serial"):
     try:
         func = getattr(_distances[backend], funcname)
     except KeyError:
-        raise_from(
-            ValueError(
-                "Function {0} not available with backend {1}; try one "
-                "of: {2}".format(funcname, backend, _distances.keys())),
-            None)
+        errmsg = (f"Function {funcname} not available with backend {backend} "
+                  f"try one of: {_distances.keys()}")
+        raise ValueError(errmsg) from None
     return func(*args, **kwargs)
 
 # serial versions are always available (and are typically used within
 # the core and topology modules)
-from .c_distances import (calc_distance_array,
+from .c_distances import (_UINT64_MAX,
+                          calc_distance_array,
                           calc_distance_array_ortho,
                           calc_distance_array_triclinic,
                           calc_self_distance_array,
@@ -201,7 +198,7 @@ def distance_array(reference, configuration, box=None, result=None,
     box : array_like, optional
         The unitcell dimensions of the system, which can be orthogonal or
         triclinic and must be provided in the same format as returned by
-        :attr:`MDAnalysis.coordinates.base.Timestep.dimensions`:\n
+        :attr:`MDAnalysis.coordinates.base.Timestep.dimensions`:
         ``[lx, ly, lz, alpha, beta, gamma]``.
     result : numpy.ndarray, optional
         Preallocated result array which must have the shape ``(n, m)`` and dtype
@@ -227,10 +224,14 @@ def distance_array(reference, configuration, box=None, result=None,
     confnum = configuration.shape[0]
     refnum = reference.shape[0]
 
+    # check resulting array will not overflow UINT64_MAX
+    if refnum * confnum > _UINT64_MAX:
+        raise ValueError(f"Size of resulting array {refnum * confnum} elements"
+                         " larger than size of maximum integer")
+
     distances = _check_result_array(result, (refnum, confnum))
     if len(distances) == 0:
         return distances
-
     if box is not None:
         boxtype, box = check_box(box)
         if boxtype == 'ortho':
@@ -269,7 +270,7 @@ def self_distance_array(reference, box=None, result=None, backend="serial"):
     box : array_like, optional
         The unitcell dimensions of the system, which can be orthogonal or
         triclinic and must be provided in the same format as returned by
-        :attr:`MDAnalysis.coordinates.base.Timestep.dimensions`:\n
+        :attr:`MDAnalysis.coordinates.base.Timestep.dimensions`:
         ``[lx, ly, lz, alpha, beta, gamma]``.
     result : numpy.ndarray, optional
         Preallocated result array which must have the shape ``(n*(n-1)/2,)`` and
@@ -299,11 +300,14 @@ def self_distance_array(reference, box=None, result=None, backend="serial"):
     """
     refnum = reference.shape[0]
     distnum = refnum * (refnum - 1) // 2
+    # check resulting array will not overflow UINT64_MAX
+    if distnum > _UINT64_MAX:
+        raise ValueError(f"Size of resulting array {distnum} elements larger"
+                         " than size of maximum integer")
 
     distances = _check_result_array(result, (distnum,))
     if len(distances) == 0:
         return distances
-
     if box is not None:
         boxtype, box = check_box(box)
         if boxtype == 'ortho':
@@ -336,7 +340,7 @@ def capped_distance(reference, configuration, max_cutoff, min_cutoff=None,
     An automatic guessing of the optimal method to calculate the distances is
     included in the function. An optional keyword for the method is also
     provided. Users can enforce a particular method with this functionality.
-    Currently brute force and periodic KDtree methods are
+    Currently brute force, grid search, and periodic KDtree methods are
     implemented.
 
     Parameters
@@ -352,9 +356,9 @@ def capped_distance(reference, configuration, max_cutoff, min_cutoff=None,
     box : array_like, optional
         The unitcell dimensions of the system, which can be orthogonal or
         triclinic and must be provided in the same format as returned by
-        :attr:`MDAnalysis.coordinates.base.Timestep.dimensions`:\n
+        :attr:`MDAnalysis.coordinates.base.Timestep.dimensions`:
         ``[lx, ly, lz, alpha, beta, gamma]``.
-    method : {'bruteforce', 'pkdtree'}, optional
+    method : {'bruteforce', 'nsgrid', 'pkdtree'}, optional
         Keyword to override the automatic guessing of the employed search
         method.
     return_distances : bool, optional
@@ -389,6 +393,7 @@ def capped_distance(reference, configuration, max_cutoff, min_cutoff=None,
     --------
     distance_array
     MDAnalysis.lib.pkdtree.PeriodicKDTree.search
+    MDAnalysis.lib.nsgrid.FastNS.search
 
 
     .. versionchanged:: 1.0.1
@@ -430,9 +435,9 @@ def _determine_method(reference, configuration, max_cutoff, min_cutoff=None,
     box : numpy.ndarray
         The unitcell dimensions of the system, which can be orthogonal or
         triclinic and must be provided in the same format as returned by
-        :attr:`MDAnalysis.coordinates.base.Timestep.dimensions`:\n
+        :attr:`MDAnalysis.coordinates.base.Timestep.dimensions`:
         ``[lx, ly, lz, alpha, beta, gamma]``.
-    method : {'bruteforce', 'pkdtree'}, optional
+    method : {'bruteforce', 'nsgrid', 'pkdtree'}, optional
         Keyword to override the automatic guessing of the employed search
         method.
 
@@ -446,7 +451,7 @@ def _determine_method(reference, configuration, max_cutoff, min_cutoff=None,
        nsgrid was temporarily removed and replaced with pkdtree due to issues
        relating to its reliability and accuracy (Issues #2919, #2229, #2345,
        #2670, #2930)
-    .. versionchanged:: 1.0.2
+    .. versionchanged:: 1.1.0
        enabled nsgrid again
     """
     methods = {'bruteforce': _bruteforce_capped,
@@ -515,7 +520,7 @@ def _bruteforce_capped(reference, configuration, max_cutoff, min_cutoff=None,
     box : numpy.ndarray, optional
         The unitcell dimensions of the system, which can be orthogonal or
         triclinic and must be provided in the same format as returned by
-        :attr:`MDAnalysis.coordinates.base.Timestep.dimensions`:\n
+        :attr:`MDAnalysis.coordinates.base.Timestep.dimensions`:
         ``[lx, ly, lz, alpha, beta, gamma]``.
     return_distances : bool, optional
         If set to ``True``, distances will also be returned.
@@ -592,7 +597,7 @@ def _pkdtree_capped(reference, configuration, max_cutoff, min_cutoff=None,
     box : numpy.ndarray, optional
         The unitcell dimensions of the system, which can be orthogonal or
         triclinic and must be provided in the same format as returned by
-        :attr:`MDAnalysis.coordinates.base.Timestep.dimensions`:\n
+        :attr:`MDAnalysis.coordinates.base.Timestep.dimensions`:
         ``[lx, ly, lz, alpha, beta, gamma]``.
     return_distances : bool, optional
         If set to ``True``, distances will also be returned.
@@ -674,7 +679,7 @@ def _nsgrid_capped(reference, configuration, max_cutoff, min_cutoff=None,
     box : numpy.ndarray (``dtype=numpy.float64``, ``shape=(n_pairs,)``), optional
         The unitcell dimensions of the system, which can be orthogonal or
         triclinic and must be provided in the same format as returned by
-        :attr:`MDAnalysis.coordinates.base.Timestep.dimensions`:\n
+        :attr:`MDAnalysis.coordinates.base.Timestep.dimensions`:
         ``[lx, ly, lz, alpha, beta, gamma]``.
     return_distances : bool, optional
         If set to ``True``, distances will also be returned.
@@ -752,7 +757,7 @@ def self_capped_distance(reference, max_cutoff, min_cutoff=None, box=None,
     An automatic guessing of the optimal method to calculate the distances is
     included in the function. An optional keyword for the method is also
     provided. Users can enforce a particular method with this functionality.
-    Currently brute force, and periodic KDtree methods are
+    Currently brute force, grid search, and periodic KDtree methods are
     implemented.
 
     Parameters
@@ -766,9 +771,9 @@ def self_capped_distance(reference, max_cutoff, min_cutoff=None, box=None,
     box : array_like, optional
         The unitcell dimensions of the system, which can be orthogonal or
         triclinic and must be provided in the same format as returned by
-        :attr:`MDAnalysis.coordinates.base.Timestep.dimensions`:\n
+        :attr:`MDAnalysis.coordinates.base.Timestep.dimensions`:
         ``[lx, ly, lz, alpha, beta, gamma]``.
-    method : {'bruteforce', 'pkdtree'}, optional
+    method : {'bruteforce', 'nsgrid', 'pkdtree'}, optional
         Keyword to override the automatic guessing of the employed search
         method.
     return_distances : bool, optional
@@ -800,13 +805,14 @@ def self_capped_distance(reference, max_cutoff, min_cutoff=None, box=None,
 
     Note
     -----
-    Currently supports brute force, and periodic KDtree search
+    Currently supports brute force, grid-based, and periodic KDtree search
     methods.
 
     See Also
     --------
     self_distance_array
     MDAnalysis.lib.pkdtree.PeriodicKDTree.search
+    MDAnalysis.lib.nsgrid.FastNS.self_search
 
 
     .. versionchanged:: 0.20.0
@@ -848,9 +854,9 @@ def _determine_method_self(reference, max_cutoff, min_cutoff=None, box=None,
     box : numpy.ndarray
         The unitcell dimensions of the system, which can be orthogonal or
         triclinic and must be provided in the same format as returned by
-        :attr:`MDAnalysis.coordinates.base.Timestep.dimensions`:\n
+        :attr:`MDAnalysis.coordinates.base.Timestep.dimensions`:
         ``[lx, ly, lz, alpha, beta, gamma]``.
-    method : {'bruteforce', 'pkdtree'}, optional
+    method : {'bruteforce', 'nsgrid', 'pkdtree'}, optional
         Keyword to override the automatic guessing of the employed search
         method.
 
@@ -921,7 +927,7 @@ def _bruteforce_capped_self(reference, max_cutoff, min_cutoff=None, box=None,
     box : numpy.ndarray, optional
         The unitcell dimensions of the system, which can be orthogonal or
         triclinic and must be provided in the same format as returned by
-        :attr:`MDAnalysis.coordinates.base.Timestep.dimensions`:\n
+        :attr:`MDAnalysis.coordinates.base.Timestep.dimensions`:
         ``[lx, ly, lz, alpha, beta, gamma]``.
     return_distances : bool, optional
         If set to ``True``, distances will also be returned.
@@ -996,7 +1002,7 @@ def _pkdtree_capped_self(reference, max_cutoff, min_cutoff=None, box=None,
     box : numpy.ndarray, optional
         The unitcell dimensions of the system, which can be orthogonal or
         triclinic and must be provided in the same format as returned by
-        :attr:`MDAnalysis.coordinates.base.Timestep.dimensions`:\n
+        :attr:`MDAnalysis.coordinates.base.Timestep.dimensions`:
         ``[lx, ly, lz, alpha, beta, gamma]``.
     return_distances : bool, optional
         If set to ``True``, distances will also be returned.
@@ -1072,7 +1078,7 @@ def _nsgrid_capped_self(reference, max_cutoff, min_cutoff=None, box=None,
     box : numpy.ndarray, optional
         The unitcell dimensions of the system, which can be orthogonal or
         triclinic and must be provided in the same format as returned by
-        :attr:`MDAnalysis.coordinates.base.Timestep.dimensions`:\n
+        :attr:`MDAnalysis.coordinates.base.Timestep.dimensions`:
         ``[lx, ly, lz, alpha, beta, gamma]``.
 
     Returns
@@ -1157,7 +1163,7 @@ def transform_RtoS(coords, box, backend="serial"):
     box : numpy.ndarray
         The unitcell dimensions of the system, which can be orthogonal or
         triclinic and must be provided in the same format as returned by
-        :attr:`MDAnalysis.coordinates.base.Timestep.dimensions`:\n
+        :attr:`MDAnalysis.coordinates.base.Timestep.dimensions`:
         ``[lx, ly, lz, alpha, beta, gamma]``.
     backend : {'serial', 'OpenMP'}, optional
         Keyword selecting the type of acceleration.
@@ -1206,7 +1212,7 @@ def transform_StoR(coords, box, backend="serial"):
     box : numpy.ndarray
         The unitcell dimensions of the system, which can be orthogonal or
         triclinic and must be provided in the same format as returned by
-        :attr:`MDAnalysis.coordinates.base.Timestep.dimensions`:\n
+        :attr:`MDAnalysis.coordinates.base.Timestep.dimensions`:
         ``[lx, ly, lz, alpha, beta, gamma]``.
     backend : {'serial', 'OpenMP'}, optional
         Keyword selecting the type of acceleration.
@@ -1270,7 +1276,7 @@ def calc_bonds(coords1, coords2, box=None, result=None, backend="serial"):
     box : numpy.ndarray, optional
         The unitcell dimensions of the system, which can be orthogonal or
         triclinic and must be provided in the same format as returned by
-        :attr:`MDAnalysis.coordinates.base.Timestep.dimensions`:\n
+        :attr:`MDAnalysis.coordinates.base.Timestep.dimensions`:
         ``[lx, ly, lz, alpha, beta, gamma]``.
     result : numpy.ndarray, optional
         Preallocated result array of dtype ``numpy.float64`` and shape ``(n,)``
@@ -1359,7 +1365,7 @@ def calc_angles(coords1, coords2, coords3, box=None, result=None,
     box : numpy.ndarray, optional
         The unitcell dimensions of the system, which can be orthogonal or
         triclinic and must be provided in the same format as returned by
-        :attr:`MDAnalysis.coordinates.base.Timestep.dimensions`:\n
+        :attr:`MDAnalysis.coordinates.base.Timestep.dimensions`:
         ``[lx, ly, lz, alpha, beta, gamma]``.
     result : numpy.ndarray, optional
         Preallocated result array of dtype ``numpy.float64`` and shape ``(n,)``
@@ -1411,7 +1417,7 @@ def calc_angles(coords1, coords2, coords3, box=None, result=None,
 @check_coords('coords1', 'coords2', 'coords3', 'coords4')
 def calc_dihedrals(coords1, coords2, coords3, coords4, box=None, result=None,
                    backend="serial"):
-    """Calculates the dihedral angles formed between quadruplets of positions
+    r"""Calculates the dihedral angles formed between quadruplets of positions
     from the four coordinate arrays `coords1`, `coords2`, `coords3`, and
     `coords4`, which must contain the same number of coordinates.
 
@@ -1461,7 +1467,7 @@ def calc_dihedrals(coords1, coords2, coords3, coords4, box=None, result=None,
     box : numpy.ndarray, optional
         The unitcell dimensions of the system, which can be orthogonal or
         triclinic and must be provided in the same format as returned by
-        :attr:`MDAnalysis.coordinates.base.Timestep.dimensions`:\n
+        :attr:`MDAnalysis.coordinates.base.Timestep.dimensions`:
         ``[lx, ly, lz, alpha, beta, gamma]``.
     result : numpy.ndarray, optional
         Preallocated result array of dtype ``numpy.float64`` and shape ``(n,)``
@@ -1525,7 +1531,7 @@ def apply_PBC(coords, box, backend="serial"):
     box : numpy.ndarray
         The unitcell dimensions of the system, which can be orthogonal or
         triclinic and must be provided in the same format as returned by
-        :attr:`MDAnalysis.coordinates.base.Timestep.dimensions`:\n
+        :attr:`MDAnalysis.coordinates.base.Timestep.dimensions`:
         ``[lx, ly, lz, alpha, beta, gamma]``.
     backend : {'serial', 'OpenMP'}, optional
         Keyword selecting the type of acceleration.
@@ -1553,3 +1559,45 @@ def apply_PBC(coords, box, backend="serial"):
         _run("triclinic_pbc", args=(coords, box), backend=backend)
 
     return coords
+
+
+@check_coords('vectors', enforce_copy=False, enforce_dtype=False)
+def minimize_vectors(vectors, box):
+    """Apply minimum image convention to an array of vectors
+
+    This function is required for calculating the correct vectors between two
+    points.  A naive approach of ``ag1.positions - ag2.positions`` will not
+    provide the minimum vectors between particles, even if all particles are
+    within the primary unit cell (box).
+
+    Parameters
+    ----------
+    vectors : numpy.ndarray
+        Vector array of shape ``(n, 3)``, either float32 or float64.  These
+        represent many vectors (such as between two particles).
+    box : numpy.ndarray
+        The unitcell dimensions of the system, which can be orthogonal or
+        triclinic and must be provided in the same format as returned by
+        :attr:`MDAnalysis.coordinates.base.Timestep.dimensions`:
+        ``[lx, ly, lz, alpha, beta, gamma]``.
+
+    Returns
+    -------
+    minimized_vectors : numpy.ndarray
+        Same shape and dtype as input.  The vectors from the input, but
+        minimized according to the size of the box.
+
+    .. versionadded:: 2.1.0
+    """
+    boxtype, box = check_box(box)
+    output = np.empty_like(vectors)
+
+    # use box which is same precision as input vectors
+    box = box.astype(vectors.dtype)
+
+    if boxtype == 'ortho':
+        _minimize_vectors_ortho(vectors, box, output)
+    else:
+        _minimize_vectors_triclinic(vectors, box.ravel(), output)
+
+    return output
